#include "StdAfx.h"
#include "NtalkClientSession.h"
#include "NTalkPacketHeader.h"
#include "AESCrypto.h"

CNtalkClientSession::CNtalkClientSession(INTalkClientEvents* pEvents) : m_pEvents(pEvents)
{
	m_pNpSession = NULL;
	m_pNtalkModel = NULL;
	m_networkError = 0;	
}

CNtalkClientSession::~CNtalkClientSession(void)
{
	DestroyChatSession();
}

BOOL CNtalkClientSession::CreateChatSession( LPCTSTR strSrvAddress, int nPort, ULONG nRoomID, LPCTSTR pszCreaterID, LPCTSTR pszUserName, LPCTSTR pszNickName, BOOL bCreation /*= FALSE*/ )
{
	USES_CONVERSION;

	BOOL bSuccess = FALSE;
	CString strStamp;

	if(m_pNtalkModel)
	{
		delete m_pNtalkModel;
		m_pNtalkModel = NULL;
	}

	m_pNtalkModel = new CNtalkModel;

//	GT_FLOG(L"CreateChatSession %s:%d:%d", strSrvAddress, nPort, nRoomID);

	if(m_pNtalkModel)
	{		
		m_pNtalkModel->SetServerInfo(strSrvAddress, nPort, nRoomID);
		m_pNtalkModel->SetUserInfo(pszCreaterID, pszUserName, pszNickName);
		m_pNtalkModel->SetCreateRoomType(bCreation);	

		if(m_pNpSession)
		{
			m_pNpSession->Delete();
			m_pNpSession = NULL;
		}

		m_pNpSession = CreateNpSession(this, true);		

		if(m_pNpSession->ConnectSession(W2A(strSrvAddress), nPort) == true)
			bSuccess = TRUE;
		else
		{
//			GT_FLOG(L"CreateChatSession Fail");
			bSuccess = FALSE;
		}
	}
	else
	{
//		GT_FLOG(L"CreateChatSession Fail");
		bSuccess = FALSE;
	}

	return bSuccess;
}

BOOL CNtalkClientSession::WSASendErrorLogging( WSABUF wsaBuf )
{
	return FALSE;
}

void CNtalkClientSession::DebugStringWSAError()
{	
}

void CNtalkClientSession::DestroyChatSession()
{
	if(m_pNpSession)
	{
		m_pNpSession->CloseSession();
		m_pNpSession->Delete();
		m_pNpSession = NULL;
	}

	if(m_pNtalkModel)
	{		
		delete m_pNtalkModel;
		m_pNtalkModel = NULL;
	}
}

BOOL CNtalkClientSession::RequestInitRoom()
{
	if( m_pNtalkModel == NULL && m_pNpSession != NULL )
		return FALSE;

	//äù 
	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen()*4 + m_pNtalkModel->GetNickNameLen()*4 + 2 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char)*(nTotalSize + 1));


	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;		

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());


	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetOsType()), 2);
	nBodyLen += 2;

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_INITROOM, nBodyLen, &szBuffer);

	
	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
//	GT_FLOG(L"[%s]:P:SND:%d:-1:ROOM_CREATE:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID());

	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RegisterChatUsers( std::vector<PINVITE_USER> vecUserList )
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	//ȭ ʴ
	int nBodyLen = 0, nTotalSize = 0, nUserListLen = 0;
	USHORT nLen = 0;
	CString strInvitedID(_T(""));

	std::vector<PINVITE_USER>::const_iterator user_iter = vecUserList.begin();
	for(; user_iter != vecUserList.end(); ++user_iter)
	{
		PINVITE_USER pUserList = *user_iter;

		CString strUserNameAndNickName;
		strUserNameAndNickName.Format(_T("%s|%s"), pUserList->pszName, pUserList->pszNickName);

		nUserListLen += wcslen(pUserList->pszID) + 1;
		nUserListLen += strUserNameAndNickName.GetLength() + 1;		
	}

	nTotalSize = 4 + 4 + nUserListLen * 4 + PACKET_HEADER_LEN;	

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char)*(nTotalSize + 1));

	//ȭ ̵
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;

	UINT nUserCount = vecUserList.size();	
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nUserCount, 4);
	nBodyLen += 4;

	for(std::vector<PINVITE_USER>::iterator iter = vecUserList.begin(); 
		iter != vecUserList.end(); iter++)
	{
		PINVITE_USER pUserList = *iter;

		CString strUserNameAndNickName;
		strUserNameAndNickName.Format(_T("%s|%s"), pUserList->pszName, pUserList->pszNickName);

		nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, pUserList->pszID);
		nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, strUserNameAndNickName.GetString());

		CString strStamp;
		LogUtil::GetCurrentTimeStamp(strStamp);		
		//GT_FLOG(L"[%s]:P:SND:%d:-1:USER_ADD:%s:PC::%s", strStamp, m_pNtalkModel->GetRoomID(), pUserList->pszID, m_pNtalkModel->GetUserID());		
	}

	nBodyLen += CNTalkPacketHeader::MakeHeader(MSNG_NE_CRS_INVITE_LIST, nBodyLen, &szBuffer);
	
	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestSendChatMsg( LPCTSTR pszMsg, CHARFORMAT2* pFont, UINT nTid, UINT nMsgSeq, BYTE MsgType)
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBufLen = 0, nTotalSize = 0;	
	int nBodyLen = 0, nMsgLen = 0, nFontSize = 0, nFontFaceNameSize = 0;

	nMsgLen = ::WideCharToMultiByte(CP_UTF8, 0, pszMsg, -1, NULL, 0, NULL, NULL);	

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen() * 4 + m_pNtalkModel->GetNickNameLen() * 4 + 2 + 2 + (nMsgLen * 4 + 1) + 4 + GetCharFormat2Len(pFont) + 4 + 4 + 1 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char)*(nTotalSize + 1));

	//ȭ ̵
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());	

	// Ŭ̾Ʈ Ÿ
	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	nBufLen = ::WideCharToMultiByte(CP_UTF8, 0, pszMsg, -1, NULL, 0, NULL, NULL);
	char *pMultiMsg = new char[nBufLen];	
	if (NULL == pMultiMsg)
	{
		SAFE_DELETE_ARR(szBuffer);
		return FALSE;
	}

	::WideCharToMultiByte(CP_UTF8, 0, pszMsg, -1, pMultiMsg, nBufLen, NULL, NULL);

	/*
	// PLAIN////////////////////////////////////////////////////////////////
	nBufLen -= 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nBufLen, 2);	
	nBodyLen += 2;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, pMultiMsg, nBufLen);		
	nBodyLen += nBufLen;
	// PLAIN////////////////////////////////////////////////////////////////
	//*/
	
	///*
	// ENC///////////////////////////////////////////////////////////////////
	// encrypt   16   	
	int nMultiple = nBufLen / 16;
	nBufLen = 16 * (nMultiple + 1);

	unsigned char* szMsgBuf = new unsigned char[nBufLen];
	memset(szMsgBuf, 0x00, sizeof(unsigned char) * nBufLen);
	int nEncSize = EncryptMsg(m_pNtalkModel->GetUserID(), pMultiMsg, szMsgBuf);	
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nEncSize, 2);	
	nBodyLen += 2;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, szMsgBuf, nEncSize);
	nBodyLen += nEncSize;
	// ENC///////////////////////////////////////////////////////////////////
	//*/

	/*
	1. UINT		cbSize;
	2. DWORD		dwMask;
	3. DWORD		dwEffects;
	4. LONG		yHeight;
	5. LONG		yOffset;
	6, 7, 8. COLORREF	crTextColor;
	9. BYTE		bCharSet;
	10. BYTE		bPitchAndFamily;
	11. WCHAR		szFaceName[LF_FACESIZE];
	*/	

	char aFontInfo[512] = {0,};
	int   index = 0;
	int   nSize = 0;

	memcpy(aFontInfo + index,&pFont->cbSize,nSize = sizeof(pFont->cbSize));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->dwMask,nSize = sizeof(pFont->dwMask));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->dwEffects,nSize = sizeof(pFont->dwEffects));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->yHeight,nSize = sizeof(pFont->yHeight));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->yOffset,nSize = sizeof(pFont->yOffset));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->crTextColor,nSize = sizeof(pFont->crTextColor));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->bCharSet,nSize = sizeof(pFont->bCharSet));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->bPitchAndFamily,nSize = sizeof(pFont->bPitchAndFamily));
	index += nSize;
 
	char aFaceName[LF_FACESIZE] = {0,};	
	nFontFaceNameSize = ::WideCharToMultiByte(CP_UTF8, 0, pFont->szFaceName, -1, NULL, 0, NULL, NULL);	
	::WideCharToMultiByte(CP_UTF8, 0, pFont->szFaceName, -1, aFaceName, nFontFaceNameSize, NULL, NULL);

	memcpy(aFontInfo + index,aFaceName,nSize = sizeof(aFaceName));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->wWeight,nSize = sizeof(pFont->wWeight));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->sSpacing,nSize = sizeof(pFont->sSpacing));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->crBackColor,nSize = sizeof(pFont->crBackColor));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->lcid,nSize = sizeof(pFont->lcid));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->dwReserved,nSize = sizeof(pFont->dwReserved));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->sStyle,nSize = sizeof(pFont->sStyle));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->wKerning,nSize = sizeof(pFont->wKerning));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->bUnderlineType,nSize = sizeof(pFont->bUnderlineType));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->bAnimation,nSize = sizeof(pFont->bAnimation));
	index += nSize;
	memcpy(aFontInfo + index,&pFont->bRevAuthor,nSize = sizeof(pFont->bRevAuthor));
	index += nSize;

 
	UINT nFontInfoLen = index;
	
	//Ʈ  
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nFontInfoLen, 4);
	nBodyLen += 4;
	//Ʈ 
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, aFontInfo, nFontInfoLen);
	nBodyLen += nFontInfoLen;

	// tid
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nTid, 4);
	nBodyLen += 4;	

	// MsgSeq
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nMsgSeq, 4);
	nBodyLen += 4;	

	// MsgType (1:Ϲݴȭ, 4:ƼĿ)
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &MsgType, 1);
	nBodyLen += 1;

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_SENDMSG, nBodyLen, &szBuffer);
	
	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:-1:MSG:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID());	
	
	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(pMultiMsg);
	SAFE_DELETE_ARR(szBuffer);
	SAFE_DELETE_ARR(szMsgBuf);

	return TRUE;
}

BOOL CNtalkClientSession::RequestFileSend(std::vector<wstring> vecReceiverIds, LPCTSTR pszFileName, UINT nFileSize, LPCTSTR pszURL, UINT nTid, UINT nMsgSeq)
{
	if (m_pNtalkModel == NULL || m_pNpSession == NULL)
		return FALSE;

	if (vecReceiverIds.size() <= 0)
		return FALSE;

	int nTotalSize = 0;	
	int nBodyLen = 0, nReceiverIdLen = 0;

	for (size_t i = 0; i < vecReceiverIds.size(); i++)
	{
		wstring receiverId = vecReceiverIds[i];
		nReceiverIdLen += receiverId.length();;
	}

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen() * 4 + m_pNtalkModel->GetNickNameLen() * 4 + 2 + 4 + 4 + 1 + 2 + nReceiverIdLen * 4 + _tcslen(pszFileName) * 4 + 4 + _tcslen(pszURL) * 4 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char)*(nTotalSize + 1));

	// ȭ ̵
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;

	// » ̵
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	
	// » ȭ
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());	

	//  Ŭ̾Ʈ Ÿ
	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	// tid
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nTid, 4);
	nBodyLen += 4;	

	// MsgSeq
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nMsgSeq, 4);
	nBodyLen += 4;	

	// MsgType (5: )
	BYTE msgType = 5;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &msgType, 1);
	nBodyLen += 1;

	//  
	USHORT nReceiverCount = vecReceiverIds.size();
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nReceiverCount, 2);
	nBodyLen += 2;

	//  ̵
	for (size_t i = 0; i < vecReceiverIds.size(); i++)
	{
		wstring receiverId = vecReceiverIds[i];
		nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, receiverId.c_str());
	}

	// ϸ
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, pszFileName);

	//  ũ
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nFileSize, 4);
	nBodyLen += 4;

	// ٿ URL
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, pszURL);

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_ATTACH_SEND, nBodyLen, &szBuffer);

	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestTyping( NTalkTyping state )
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen() * 4 + m_pNtalkModel->GetNickNameLen() * 4 + 2 + 1 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());

	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	unsigned char cState;
	if (typeTyping == state)
	{
		cState = (unsigned char)1;
	}
	else
	{
		cState = (unsigned char)0;
	}

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &cState, 1);
	nBodyLen += 1;

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_TYPING, nBodyLen, &szBuffer);		

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:-1:TYPING:%s:PC::%d", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID(), (int)cState);		
	
	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestChatJoin()
{	
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen() * 4 + m_pNtalkModel->GetNickNameLen() * 4 + 2 + PACKET_HEADER_LEN;
	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;		

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());

	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_JOIN, nBodyLen, &szBuffer);

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:-1:USER_JOIN:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID());
	
	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestForceChatReJoin()
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen() * 4 + m_pNtalkModel->GetNickNameLen() * 4 + 2 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;		

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());

	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_FORCE_REJOIN, nBodyLen, &szBuffer);	

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:-1:USER_REJOIN:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID());		
	
	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestChatLeave()
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen()*4 + m_pNtalkModel->GetNickNameLen()*4 + 2 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;		

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());

	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_LEAVE, nBodyLen, &szBuffer);	

	
	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:-1:USER_LEAVE:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID());		

	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestForceLeave( LPCTSTR pszUserID, LPCTSTR pszUserName, LPCTSTR pszNickName )
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBodyLen = 0, nTotalSize = 0;
	int nUserIdLen = 0, nUserNameLen = 0, nNickNameLen = 0;

	nUserIdLen = ::WideCharToMultiByte(CP_UTF8, 0, pszUserID, -1, NULL, 0, NULL, NULL);
	nUserNameLen = ::WideCharToMultiByte(CP_UTF8, 0, pszUserName, -1, NULL, 0, NULL, NULL);
	nNickNameLen = ::WideCharToMultiByte(CP_UTF8, 0, pszNickName, -1, NULL, 0, NULL, NULL);

	CString strForceLeaveUser;
	strForceLeaveUser.Format(_T("%s|%s"), pszUserName, pszNickName);

	nTotalSize = 4 + m_pNtalkModel->GetUserIDLen()*4 + m_pNtalkModel->GetNickNameLen()*4 + 2 
		+ nUserIdLen*4 + strForceLeaveUser.GetLength()*4 +  PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;		

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetNickName());

	uint16_t nOsType = 1;
	memcpy( szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, sizeof(uint16_t) );
	nBodyLen += sizeof(uint16_t);

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, pszUserID);
	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, strForceLeaveUser.GetString());

	nBodyLen += CNTalkPacketHeader::MakeHeader(REQ_NE_CRS_FORCELEAVE, nBodyLen, &szBuffer);	

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:-1:USER_FORCE_LEAVE:%s:PC:%s", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID(), pszUserID);
	
	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

//nResult: 0(init), -1(m_pNtalkModel==NULL), -2(m_pNpSession==NULL), -3(new fail), 
//         1(call SendRawData)
BOOL CNtalkClientSession::RequestAckRecvMsg( UINT nMsgSeq, UINT nTid, INT& nReason )
{
    nReason = 0;
    if( m_pNtalkModel == NULL){
        nReason = -1;
		return FALSE;
    }
    if(m_pNpSession == NULL )
    {
        nReason = -2;
        return FALSE;
    }

	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + 4 + m_pNtalkModel->GetUserIDLen() * 4 + 4 + 2 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
    {
        nReason = -3;
		return FALSE;
    }
	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;			

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nMsgSeq, 4);
	nBodyLen += 4;

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nTid, 4);
	nBodyLen += 4;

	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	nBodyLen += CNTalkPacketHeader::MakeHeader(ACK_NE_CRS_RECVMSG, nBodyLen, &szBuffer);	

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:%d:MSG_RELAY_ACK:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), nMsgSeq, m_pNtalkModel->GetUserID());	
	
	if( m_pNpSession != NULL )
    {
		m_pNpSession->SendRawData(szBuffer, nBodyLen);
    }
    else
    {
        nReason = -2;
        SAFE_DELETE_ARR(szBuffer);
        return FALSE;
    }

    SAFE_DELETE_ARR(szBuffer);
	return TRUE;
}

BOOL CNtalkClientSession::RequestAckRecvFile( UINT nMsgSeq, UINT nTid )
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + 4 + m_pNtalkModel->GetUserIDLen() * 4 + 4 + 2 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;			

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nMsgSeq, 4);
	nBodyLen += 4;

	nBodyLen += MemcpyWideCharsWithSize(szBuffer + PACKET_HEADER_LEN + nBodyLen, m_pNtalkModel->GetUserID());

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nTid, 4);
	nBodyLen += 4;

	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	nBodyLen += CNTalkPacketHeader::MakeHeader(ACK_NE_CRS_RECVFILE, nBodyLen, &szBuffer);	

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:%d:MSG_RELAY_ACK:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), nMsgSeq, m_pNtalkModel->GetUserID());	

	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestAckMsgResult( UINT nMsgSeq, UINT nTid, UINT nResultSeq )
{
	if( m_pNtalkModel == NULL || m_pNpSession == NULL )
		return FALSE;

	int nBodyLen = 0, nTotalSize = 0;

	nTotalSize = 4 + 4 + 4 + 2 + 4 + PACKET_HEADER_LEN;

	char* szBuffer = new char[nTotalSize + 1];
	if (NULL == szBuffer)
		return FALSE;

	ZeroMemory(szBuffer, sizeof(char) * (nTotalSize + 1));

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &(m_pNtalkModel->GetRoomID()), 4);
	nBodyLen += 4;			

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nMsgSeq, 4);
	nBodyLen += 4;	

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nTid, 4);
	nBodyLen += 4;

	USHORT nOsType = 1;
	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nOsType, 2);
	nBodyLen += 2;

	memcpy(szBuffer + PACKET_HEADER_LEN + nBodyLen, &nResultSeq, 4);
	nBodyLen += 4;

	nBodyLen += CNTalkPacketHeader::MakeHeader(ACK_NE_CRS_MSG_RESULT, nBodyLen, &szBuffer);	

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:%d:MSG_RESULT_ACK:%s:%s::%d", strStamp, m_pNtalkModel->GetRoomID(), nMsgSeq, m_pNtalkModel->GetUserID(), nOsType == 1 ? L"PC" : L"MOBILE", nResultSeq);
	
	if( m_pNpSession != NULL )
		m_pNpSession->SendRawData(szBuffer, nBodyLen);

	SAFE_DELETE_ARR(szBuffer);

	return TRUE;
}

BOOL CNtalkClientSession::RequestSendPing()
{
	if(m_pNpSession == NULL )
		return false;

	return m_pNpSession->SendNpPacket_SessionControlPacket(0x81, false);
}

void CNtalkClientSession::SetNickname( LPCTSTR pszNickName )
{
	if( m_pNtalkModel == NULL )
		return;

	m_pNtalkModel->SetNickName(pszNickName);
}

int CNtalkClientSession::GetNetworkError()
{
	return m_networkError;
}

void CNtalkClientSession::SetNetworkError( int error )
{
	m_networkError = error;
}

void CNtalkClientSession::ServerSessionEnd()
{
	//GT_FLOG_NOARG(L"ServerSessionEnd");
	if(m_pEvents)
		m_pEvents->NtalkClient_OnServerSessionEnd(this);
}

void CALLBACK CNtalkClientSession::OnNpSession_Connect( INpSession *pSession, int nSocketError )
{
	if( m_pNtalkModel == NULL )
		return;

	if(nSocketError)
		ReleaseSession();

	if(m_pEvents)
		m_pEvents->NtalkClient_OnCreateChatSession(this, (m_pNtalkModel->GetCreateRoomType() == TRUE ? typeChatCreate : typeChatJoin), nSocketError ? createSessionError_Connect : createSessionError_None);
}

void CALLBACK CNtalkClientSession::OnNpSession_Close( INpSession *pSession, int nSocketError, bool bRecvInvalidPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:SND:%d:-1:CLOSE:%s:PC:FAIL:%d", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID(), nSocketError);

	ReleaseSession();
	if(m_pEvents)
		m_pEvents->NtalkClient_OnServerSessionEnd(this);
}

void CALLBACK CNtalkClientSession::OnNpSession_RecvPacket( INpSession *pSession, INpPacket * pPacket )
{
	if( !pPacket->IsM6V2() )
	{
		BYTE byte = *pPacket->RawHeader();
		if( byte == 0xf81 )
		{
			// Receive Ping & Send Pong
			pSession->SendNpPacket_SessionControlPacket( 0x82, false );
		}
		else
		{
			// Receive Pong
			RecvPong();
		}

		return;
	}

	switch(pPacket->M6V2()->usCMMD)
	{
	case PUSH_NE_CRS_RECVMSG:			// ȭ ˸
		RecvPushRecvMsg(pPacket);
		break;
	case PUSH_NE_CRS_TYPING:			//   Է  ˸
		RecvPushTyping(pPacket);
		break;
	case RES_NE_CRS_JOIN:				// ȭû 
		RecvResJoin(pPacket);
		break;
	case RES_NE_CRS_FORCE_REJOIN:		// ȭû 
		RecvResForceReJoin(pPacket);
		break;
	case RES_NE_CRS_FORCELEAVE:
		//RecvResForceLeave(pPacket);
		break;
	case PUSH_NE_FORCE_LEAVE:
		RecvPushForceLeave(pPacket);
		break;
	case PUSH_NE_SESSION_CLOSE:			// Duplicated Session is closed by chat server.
		RecvPushSessionClose(pPacket);
		break;
	case PUSH_NE_CRS_NEWJOINER:			// ȭ˸
		RecvPushNewJoiner(pPacket);
		break;
	case PUSH_NE_CRS_REJOINER:			// ȭ   ˸
		RecvPushForceReJoiner(pPacket);
		break;
	case PUSH_NE_CRS_LEAVEJOINER:		// ȭ  ˸
		RecvPushLeaveJoiner(pPacket);
		break;
	case ACK_NE_CRS_SENDMSG:			// ȭ ۿ   
		RecvAckSendMsg(pPacket);
		break;
	case RES_NE_CRS_INITROOM:			// ȭ  
		RecvResInitRoom(pPacket);
		break;
	case RES_NE_CRS_INVITE_LIST:		// ȭ   
		RecvResInviteList(pPacket);
		break;
	case PUSH_NE_CRS_ROOM_END:			// ȭ  ˸
		RecvPushRoomEnd(pPacket);
		break;
    case PUSH_NE_CRS_SESSION_END:       // ȭ   ˸
        RecvPushSessionEnd(pPacket);
        break;
	case NE_CRS_NOT_READY:				// ä  غ 
		RecvServerNotReady();
		break;
	case PUSH_NE_CRS_MSG_FAIL:			// ȭ   ˸
		RecvPushMsgFail(pPacket);
		break;
	case PUSH_NE_CRS_MSG_RESULT:		// ȭ  
		RecvPushMsgResult(pPacket);
		break;
	case ACK_NE_CRS_ATTACH_RESULT:		//   ˸   
		RecvAckSendFile(pPacket);
		break;
	case PUSH_NE_CRS_ATTACH_RECV:		//   ˸
		RecvPushRecvFile(pPacket);
		break;
	}
}

void CALLBACK CNtalkClientSession::OnNpSession_SendReady( INpSession *pSession )
{

}

int CNtalkClientSession::MemcpyWideCharsWithSize( char * pBuffer,LPCWSTR pWString )
{
	if(pBuffer == NULL)
		return 0;
	if(pWString == NULL)
		return 0;

	int nBytes = 0;

	USHORT	nLen = ::WideCharToMultiByte(CP_UTF8, 0, pWString, -1, NULL, 0, NULL, NULL); 
	char * pCString = new char[nLen];
	if (NULL == pCString)
		return 0;

	memset(pCString, 0, nLen);

	::WideCharToMultiByte(CP_UTF8, 0, pWString, -1, pCString, nLen, NULL, NULL);

	nLen -= 1;
	memcpy(pBuffer, &nLen, sizeof(nLen));
	nBytes += sizeof(nLen);

	memcpy(pBuffer + nBytes, pCString, nLen);
	SAFE_DELETE_ARR(pCString);
	nBytes += nLen;

	return nBytes;
}

int CNtalkClientSession::GetCharFormat2Len( CHARFORMAT2* pFont )
{
	int nFontFaceNameSize = 0, nFontSize = 0;
	nFontFaceNameSize = ::WideCharToMultiByte(CP_UTF8, 0, pFont->szFaceName, -1, NULL, 0, NULL, NULL);
	nFontSize = sizeof(pFont->cbSize) + sizeof(pFont->dwMask) + sizeof(pFont->dwEffects) + 
		sizeof(pFont->yHeight) + sizeof(pFont->yOffset) + sizeof(pFont->crTextColor) + 
		sizeof(pFont->bCharSet) + sizeof(pFont->bPitchAndFamily) + nFontFaceNameSize * 4 + 
		sizeof(pFont->wWeight) + sizeof(pFont->sSpacing) + sizeof(pFont->crBackColor) + 
		sizeof(pFont->lcid) + sizeof(pFont->dwReserved) + sizeof(pFont->sStyle) + 
		sizeof(pFont->wKerning) + sizeof(pFont->bUnderlineType) + sizeof(pFont->bAnimation) + 
		sizeof(pFont->bRevAuthor);

	return nFontSize;
}

void CNtalkClientSession::RecvPushRecvFile( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	RES_RECVFILE stResRecvFile;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stResRecvFile, 0x00, sizeof(RES_RECVFILE));
	
	const char* iterator = buffer;

	TCHAR* pszRecvSenderID = NULL;
	TCHAR* pszRecvSenderNick = NULL;
	TCHAR* pszRecvFileName = NULL;
	TCHAR* pszRecvFileURL = NULL;

	// ȭ ̵
	if (ReadRoomID(nBodyLength, &stResRecvFile.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	//   ̵
	if (ReadSendUserID(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszSendUserID, -1, NULL, 0);
	pszRecvSenderID = new TCHAR[nBufLen];
	if (NULL == pszRecvSenderID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszSendUserID, -1, pszRecvSenderID, nBufLen);
	pszRecvSenderID[nBufLen - 1] = _T('\0'); 

	//   г
	if (ReadSenderNickName(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszSenderNickName, -1, NULL, 0);
	pszRecvSenderNick = new TCHAR[nBufLen];	
	if (NULL == pszRecvSenderNick)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszSenderNickName, -1, pszRecvSenderNick, nBufLen);
	pszRecvSenderNick[nBufLen - 1] = _T('\0'); 

	//  Ŭ̾Ʈ Ÿ
	if (ReadOSType(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	//  ð
	if (ReadTime(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	// Ʈ ȣ
	if (ReadTid(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	// ȭ Ϸùȣ
	if (ReadMessageSeq(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	// ȭ Ÿ( ̹Ƿ  '5' ̴.)
	if (ReadMsgType(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	// ϸ
	if (ReadFileName(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszFileName, -1, NULL, 0);
	pszRecvFileName = new TCHAR[nBufLen];	
	if (NULL == pszRecvFileName)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszFileName, -1, pszRecvFileName, nBufLen);
	pszRecvFileName[nBufLen - 1] = _T('\0'); 

	//  ũ
	if (ReadFileSize(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	// ٿ URL
	if (ReadFileURL(nBodyLength, stResRecvFile, &iterator) == FALSE)
		goto DeleteMemory;

	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszURL, -1, NULL, 0);
	pszRecvFileURL = new TCHAR[nBufLen];
	if (NULL == pszRecvFileURL)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResRecvFile.pszURL, -1, pszRecvFileURL, nBufLen);
	pszRecvFileURL[nBufLen - 1] = _T('\0'); 

	if (m_pEvents)
		m_pEvents->NtalkClient_OnReceiveFile(this, 
											stResRecvFile.nRoomID,
											pszRecvSenderID,
											pszRecvSenderNick,
											stResRecvFile.nOsType, 
											stResRecvFile.nTime,
											pszRecvFileName,
											stResRecvFile.nFileSize, 
											pszRecvFileURL, 
											stResRecvFile.nMsgSeq,
											stResRecvFile.nTid,
											stResRecvFile.msgType);

DeleteMemory:
	SAFE_DELETE_ARR(stResRecvFile.pszSendUserID);
	SAFE_DELETE_ARR(stResRecvFile.pszSenderNickName);
	SAFE_DELETE_ARR(stResRecvFile.pszFileName);
	SAFE_DELETE_ARR(stResRecvFile.pszURL);
	SAFE_DELETE_ARR(pszRecvSenderID);
	SAFE_DELETE_ARR(pszRecvSenderNick);
	SAFE_DELETE_ARR(pszRecvFileName);
	SAFE_DELETE_ARR(pszRecvFileURL);
}

void CNtalkClientSession::RecvPushRecvMsg( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;
	CString strDbg;	
	RES_RECVMSG stResRecvMsg;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stResRecvMsg, 0x00, sizeof(RES_RECVMSG));
	
	const char* iterator = buffer;
	
	TCHAR* pszRecvSenderID = NULL;
	TCHAR* pszRecvSenderNick = NULL;
	TCHAR* pszRecvMsg = NULL;	

	if (ReadRoomID(nBodyLength, &stResRecvMsg.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (ReadSendUserID(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;

	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResRecvMsg.pszSendUserID, -1, NULL, 0);
	pszRecvSenderID = new TCHAR[nBufLen];
	if (NULL == pszRecvSenderID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResRecvMsg.pszSendUserID, -1, pszRecvSenderID, nBufLen);
	pszRecvSenderID[nBufLen - 1] = _T('\0'); 
	
	if (ReadSenderNickName(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResRecvMsg.pszSenderNickName, -1, NULL, 0);
	pszRecvSenderNick = new TCHAR[nBufLen];	
	if (NULL == pszRecvSenderNick)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResRecvMsg.pszSenderNickName, -1, pszRecvSenderNick, nBufLen);
	pszRecvSenderNick[nBufLen - 1] = _T('\0'); 

	if (ReadOSType(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;

	if (ReadTime(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;

	if (ReadMessage(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;

	/*
	// PLAIN//////////////////////////////////////////////////
	TCHAR* pszRecvMsg = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResRecvMsg.pszMsg, -1, NULL, 0);	
	pszRecvMsg = new TCHAR[nBufLen];
	if (NULL == pszRecvMsg)
		goto DeleteMemory;
	
	MultiByteToWideChar(CP_UTF8, 0, stResRecvMsg.pszMsg, -1, pszRecvMsg, nBufLen);
	pszRecvMsg[nBufLen - 1] = _T('\0');
	// PLAIN//////////////////////////////////////////////////
	//*/
	
	
	///*
	// ENC/////////////////////////////////////////////////////
	char* pszPlainTxt = new char[stResRecvMsg.nMsgLen + 1];	
	DecryptMsg(pszRecvSenderID, (unsigned char*)stResRecvMsg.pszMsg, stResRecvMsg.nMsgLen, pszPlainTxt);	
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, pszPlainTxt, -1, NULL, 0);
	pszRecvMsg = new TCHAR[nBufLen];
	if (NULL == pszRecvMsg)
		goto DeleteMemory;
	MultiByteToWideChar(CP_UTF8, 0, pszPlainTxt, -1, pszRecvMsg, nBufLen);
	// ENC/////////////////////////////////////////////////////
	//*/

	if (ReadMessageSeq(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;

	//    ĽϿ Ʒ Ʈ  Ѵ. 
	CHARFORMAT2 * pCharFormat = NULL;
	if (ReadFonts(nBodyLength, stResRecvMsg, &iterator, &pCharFormat) == FALSE)
		goto DeleteMemory;

	if (ReadTid(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;

	if (ReadMsgType(nBodyLength, stResRecvMsg, &iterator) == FALSE)
		goto DeleteMemory;

	
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:%d:MSG_RELAY:%s:PC:SUCC", strStamp, m_pNtalkModel->GetRoomID(), stResRecvMsg.nMsgSeq, pszRecvSenderID);
	if(m_pEvents)
		m_pEvents->NtalkClient_OnReceiveChatMsg(this, 
												stResRecvMsg.nRoomID,
												pszRecvSenderID,
												pszRecvSenderNick,
												stResRecvMsg.nOsType, 
												stResRecvMsg.nTime,
												pszRecvMsg,
												stResRecvMsg.nMsgSeq, 
												pCharFormat, 
												stResRecvMsg.nTid,
												stResRecvMsg.msgType);

DeleteMemory:
	SAFE_DELETE_ARR(stResRecvMsg.pszMsg);
	SAFE_DELETE_ARR(stResRecvMsg.pszSenderNickName);
	SAFE_DELETE_ARR(pszRecvMsg)
	SAFE_DELETE_ARR(pszRecvSenderID)
	SAFE_DELETE_ARR(pszRecvSenderNick)	
	SAFE_DELETE_ARR(pszPlainTxt);
}

BOOL CNtalkClientSession::Read4Byte( int &nTotalRcv, void *result, const char** iterator )
{
	if (nTotalRcv < 4 || NULL == result || NULL == iterator) return FALSE;

	memcpy(result, *iterator, 4);
	*iterator += 4;
	nTotalRcv -= 4;
	return TRUE;
}

BOOL CNtalkClientSession::Read2Byte( int &nTotalRcv, void *result, const char** iterator )
{
	if (nTotalRcv < 2 || NULL == result || NULL == iterator) return FALSE;

	memcpy(result, *iterator, 2);
	*iterator += 2;
	nTotalRcv -= 2;
	return TRUE;
}

BOOL CNtalkClientSession::Read1Byte( int &nTotalRcv, void *result, const char** iterator )
{
	if (nTotalRcv < 1 || NULL == result || NULL == iterator) return FALSE;

	memcpy(result, *iterator, 1);
	*iterator += 1;
	nTotalRcv -= 1;
	return TRUE;
}

BOOL CNtalkClientSession::ReadString( int& nTotalRcv, char** ppString, const char** iterator, int length )
{
	if (nTotalRcv < length) return FALSE;

	(*ppString) = new char[length + 1];
	if (NULL == (*ppString))
		return FALSE;

	memcpy((*ppString), *iterator, length);
	(*ppString)[length] = '\0';

	*iterator += length;
	nTotalRcv -= length;

	return TRUE;
}

BOOL CNtalkClientSession::ReadSendUserID( int &nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	if (Read2Byte(nTotalRcv, &stResRecvMsg.nSendUserLen, iterator) == TRUE)
	{
		return ReadString(nTotalRcv, &stResRecvMsg.pszSendUserID, iterator, stResRecvMsg.nSendUserLen);
	}

	return FALSE;
}

BOOL CNtalkClientSession::ReadRoomID( int &nTotalRcv, void *nRoomID, const char** iterator )
{
	return Read4Byte(nTotalRcv, nRoomID, iterator);
}

BOOL CNtalkClientSession::ReadSenderNickName( int &nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	if (Read2Byte(nTotalRcv, &stResRecvMsg.nSenderNicknameLen, iterator) == TRUE)
	{
		return ReadString(nTotalRcv, &stResRecvMsg.pszSenderNickName, iterator, stResRecvMsg.nSenderNicknameLen);
	}

	return FALSE;
}

BOOL CNtalkClientSession::ReadSendUserID( int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	if (Read2Byte(nTotalRcv, &stResRecvFile.nSendUserLen, iterator) == TRUE)
	{
		return ReadString(nTotalRcv, &stResRecvFile.pszSendUserID, iterator, stResRecvFile.nSendUserLen);
	}

	return FALSE;
}

BOOL CNtalkClientSession::ReadSenderNickName( int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	if (Read2Byte(nTotalRcv, &stResRecvFile.nSenderNicknameLen, iterator) == TRUE)
	{
		return ReadString(nTotalRcv, &stResRecvFile.pszSenderNickName, iterator, stResRecvFile.nSenderNicknameLen);
	}

	return FALSE;
}

BOOL CNtalkClientSession::ReadFileName( int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	if (Read2Byte(nTotalRcv, &stResRecvFile.nFileNameLen, iterator) == TRUE)
	{
		return ReadString(nTotalRcv, &stResRecvFile.pszFileName, iterator, stResRecvFile.nFileNameLen);
	}

	return FALSE;
}

BOOL CNtalkClientSession::ReadFileSize( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	return Read4Byte(nTotalRcv, &stResRecvFile.nFileSize, iterator);
}

BOOL CNtalkClientSession::ReadFileURL( int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	if (Read2Byte(nTotalRcv, &stResRecvFile.nURLLen, iterator) == TRUE)
	{
		return ReadString(nTotalRcv, &stResRecvFile.pszURL, iterator, stResRecvFile.nURLLen);
	}

	return FALSE;
}

BOOL CNtalkClientSession::ReadOSType( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	return Read2Byte(nTotalRcv, &stResRecvMsg.nOsType, iterator);
}

BOOL CNtalkClientSession::ReadOSType( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	return Read2Byte(nTotalRcv, &stResRecvFile.nOsType, iterator);
}

BOOL CNtalkClientSession::ReadTime( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	return Read4Byte(nTotalRcv, &stResRecvMsg.nTime, iterator);
}

BOOL CNtalkClientSession::ReadTime( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	return Read4Byte(nTotalRcv, &stResRecvFile.nTime, iterator);
}

BOOL CNtalkClientSession::ReadMessage( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	if (Read2Byte(nTotalRcv, &stResRecvMsg.nMsgLen, iterator) == TRUE)
	{
		return ReadString(nTotalRcv, &stResRecvMsg.pszMsg, iterator, stResRecvMsg.nMsgLen);
	}

	return FALSE;
}

BOOL CNtalkClientSession::ReadMessageSeq( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	return Read4Byte(nTotalRcv, &stResRecvMsg.nMsgSeq, iterator);
}

BOOL CNtalkClientSession::ReadMessageSeq( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	return Read4Byte(nTotalRcv, &stResRecvFile.nMsgSeq, iterator);
}

BOOL CNtalkClientSession::ReadTid( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	return Read4Byte(nTotalRcv, &stResRecvMsg.nTid, iterator);
}

BOOL CNtalkClientSession::ReadTid( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	return Read4Byte(nTotalRcv, &stResRecvFile.nTid, iterator);
}

BOOL CNtalkClientSession::ReadMsgType( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator )
{
	return Read1Byte(nTotalRcv, &stResRecvMsg.msgType, iterator);
}

BOOL CNtalkClientSession::ReadMsgType( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator )
{
	return Read1Byte(nTotalRcv, &stResRecvFile.msgType, iterator);
}

BOOL CNtalkClientSession::ReadFonts( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator, CHARFORMAT2** pCharFormat )
{
	if (Read4Byte(nTotalRcv, &stResRecvMsg.nFontInfoSize, iterator) == FALSE)
	{
		return FALSE;
	}

	if(stResRecvMsg.nFontInfoSize == 0)
	{
		return TRUE;
	}

	if((UINT)nTotalRcv < stResRecvMsg.nFontInfoSize)
	{
		return FALSE;
	}

	stResRecvMsg.pFontInfo = new char[stResRecvMsg.nFontInfoSize + 1];
	if (NULL == stResRecvMsg.pFontInfo)
		return FALSE;

	memcpy(stResRecvMsg.pFontInfo, *iterator, stResRecvMsg.nFontInfoSize);
	stResRecvMsg.pFontInfo[stResRecvMsg.nFontInfoSize] = '\0';
	*iterator += stResRecvMsg.nFontInfoSize;
	nTotalRcv -= stResRecvMsg.nFontInfoSize;


	*pCharFormat = new CHARFORMAT2;
	if (NULL == pCharFormat)
	{
		SAFE_DELETE_ARR(stResRecvMsg.pFontInfo);
		return FALSE;
	}

	memset(*pCharFormat,0,sizeof(CHARFORMAT2));

	int  index = 0;
	int  nSize = 0;

	memcpy(&(*pCharFormat)->cbSize,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->cbSize));
	index += nSize;
	memcpy(&(*pCharFormat)->dwMask,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->dwMask));
	index += nSize;
	memcpy(&(*pCharFormat)->dwEffects,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->dwEffects));
	index += nSize;
	memcpy(&(*pCharFormat)->yHeight,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->yHeight));
	index += nSize;
	memcpy(&(*pCharFormat)->yOffset,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->yOffset));
	index += nSize;
	memcpy(&(*pCharFormat)->crTextColor,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->crTextColor));
	index += nSize;
	memcpy(&(*pCharFormat)->bCharSet,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->bCharSet));
	index += nSize;
	memcpy(&(*pCharFormat)->bPitchAndFamily,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->bPitchAndFamily));
	index += nSize;

	char szFaceName[LF_FACESIZE] = {0};
	memcpy(&szFaceName, stResRecvMsg.pFontInfo + index, nSize = sizeof(szFaceName));
	MultiByteToWideChar(CP_UTF8, 0, szFaceName, -1, (*pCharFormat)->szFaceName, sizeof((*pCharFormat)->szFaceName));
	index += nSize;

	memcpy(&(*pCharFormat)->wWeight,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->wWeight));
	index += nSize;
	memcpy(&(*pCharFormat)->sSpacing,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->sSpacing));
	index += nSize;
	memcpy(&(*pCharFormat)->crBackColor,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->crBackColor));
	index += nSize;
	memcpy(&(*pCharFormat)->lcid,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->lcid));
	index += nSize;
	memcpy(&(*pCharFormat)->dwReserved,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->dwReserved));
	index += nSize;
	memcpy(&(*pCharFormat)->sStyle,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->sStyle));
	index += nSize;
	memcpy(&(*pCharFormat)->wKerning,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->wKerning));
	index += nSize;
	memcpy(&(*pCharFormat)->bUnderlineType,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->bUnderlineType));
	index += nSize;
	memcpy(&(*pCharFormat)->bAnimation,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->bAnimation));
	index += nSize;
	memcpy(&(*pCharFormat)->bRevAuthor,stResRecvMsg.pFontInfo + index, nSize = sizeof((*pCharFormat)->bRevAuthor));
	index += nSize; 

	return TRUE;
}

void CNtalkClientSession::RecvPushTyping( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strDbg;	
	RES_TYPING stResTyping;
	int nBufLen = 0;
	CString strStamp;

	const char* buffer;	
	memset(&stResTyping, 0x00, sizeof(RES_TYPING));
	
	buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();

	const char* iterator = buffer;
	if (Read4Byte(nBodyLength, &stResTyping.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResTyping.nUserLen, &iterator) == FALSE)
		goto DeleteMemory;

	if (ReadString(nBodyLength, &stResTyping.pszUserID, &iterator, stResTyping.nUserLen) == FALSE)
		goto DeleteMemory;
	if (Read2Byte(nBodyLength, &stResTyping.nNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;

	TCHAR* pszRecvID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResTyping.pszUserID, -1, NULL, 0);
	pszRecvID = new TCHAR[nBufLen];
	if (NULL == pszRecvID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResTyping.pszUserID, -1, pszRecvID, nBufLen);
	pszRecvID[nBufLen - 1] = _T('\0');

	if (ReadString(nBodyLength, &stResTyping.pszNickName, &iterator, stResTyping.nNicknameLen) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResTyping.nOsType, &iterator) == FALSE)
		goto DeleteMemory;

	TCHAR* pszRecvNick = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResTyping.pszNickName, -1, NULL, 0);
	pszRecvNick = new TCHAR[nBufLen];
	if (NULL == pszRecvNick)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResTyping.pszNickName, -1, pszRecvNick, nBufLen);
	pszRecvNick[nBufLen - 1] = _T('\0');

	if (nBodyLength < 1) goto DeleteMemory;
	memcpy(&stResTyping.state, iterator, 1);
	iterator += 1;				


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:TYPING_ALARM:%s:PC:SUCC:%d", strStamp, m_pNtalkModel->GetRoomID(), pszRecvID, (int)stResTyping.state);
	if(m_pEvents)
	{		
		m_pEvents->NtalkClient_OnReceiveTypingMsg(this, stResTyping.nRoomID, pszRecvID, pszRecvNick, (int)stResTyping.nOsType, (NTalkTyping)(int)stResTyping.state);
	}

DeleteMemory:
	SAFE_DELETE_ARR(stResTyping.pszUserID);
	SAFE_DELETE_ARR(stResTyping.pszNickName);
	SAFE_DELETE_ARR(pszRecvID);
	SAFE_DELETE_ARR(pszRecvNick);
	
}

void CNtalkClientSession::RecvResJoin( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;
	CString strResult;
	CString strDbg;
	RES_JOIN stResJoin;
	USHORT nUserIdLen, nUserNameLen;
	char *pszUserID = NULL;
	char* pszUserName = NULL; //username+nickname
	std::vector<PINVITE_USER> vecUserList;
	TCHAR* pwszUserName = NULL;
	USHORT nUserCount = 0;
	UINT nJoinNum = 0;
	UINT nLastMsgNum = 0;

	const char* buffer = pPacket->BodyData();	
	int nBodyLength = pPacket->BodyLength();
	memset(&stResJoin, 0x00, sizeof(RES_JOIN));

	const char* iterator = buffer;

	if (Read4Byte(nBodyLength, &stResJoin.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;	

	if (Read2Byte(nBodyLength, &stResJoin.wResCode, &iterator) == FALSE)
		goto DeleteMemory;	

	if( stResJoin.wResCode == chatJoinError_NoInviteUser || stResJoin.wResCode == chatJoinError_NoExist  )
	{
		LogUtil::GetCurrentTimeStamp(strStamp);
		//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_JOIN:%s:PC:FAIL", strStamp, stResJoin.nRoomID, m_pNtalkModel->GetUserID());

		if(m_pEvents)
			m_pEvents->NtalkClient_OnChatJoin(this, (NTalkChatJoinErrorCode)stResJoin.wResCode, stResJoin.nRoomID, vecUserList, -1, nJoinNum, nLastMsgNum);
	}

	//	usercount | idlen | id | usernamelen | username+nickname| idlen | id|...
	//	1. ü ο ù° ̵
	//	usercount | idlen
	//	2. ù° ̵ ι° ̵ 
	//	id | idlen
	//	3. ...
	//	id | idlen
	//	4.  ̵
	//	id
	if (Read2Byte(nBodyLength, &nUserCount, &iterator) == FALSE)
		goto DeleteMemory;	

	for(int i = 0; i < nUserCount; i++)
	{
		PINVITE_USER pUserList = new INVITE_USER;
		if (NULL == pUserList)
			goto DeleteMemory;

		if (Read2Byte(nBodyLength, &nUserIdLen, &iterator) == FALSE)
			goto DeleteMemory;	

		// id
		if (ReadString(nBodyLength, &pszUserID, &iterator, nUserIdLen) == FALSE)
			goto DeleteMemory;		
		int nBufLen = MultiByteToWideChar(CP_UTF8, 0, pszUserID, -1, NULL, 0);		
		pUserList->pszID = new TCHAR[nBufLen];
		if (NULL == pUserList->pszID)
		{
			delete pUserList;
			goto DeleteMemory;
		}

		MultiByteToWideChar(CP_UTF8, 0, pszUserID, -1, pUserList->pszID, nBufLen);
		pUserList->pszID[nBufLen - 1] = _T('\0');

		// username + nickname
		if (Read2Byte(nBodyLength, &nUserNameLen, &iterator) == FALSE)
			goto DeleteMemory;

		//  username+nickname Ŭ̾Ʈ  ãƼ ϱ⶧..׳ skip....		
		if (ReadString(nBodyLength, &pszUserName, &iterator, nUserNameLen) == FALSE)
		{
			delete pUserList;
			goto DeleteMemory;
		}

		nBufLen = MultiByteToWideChar(CP_UTF8, 0, pszUserName, -1, NULL, 0);
		pwszUserName = new TCHAR[nBufLen];
		if (NULL == pwszUserName)
		{
			SAFE_DELETE_ARR(pUserList->pszID);
			delete pUserList;
			goto DeleteMemory;
		}

		MultiByteToWideChar(CP_UTF8, 0, pszUserName, -1, pwszUserName, nBufLen);		
		pwszUserName[nBufLen - 1] = _T('\0');

		CString strUserName(pwszUserName);
		int nPos = strUserName.Find(_T('|'));
		if(nPos != -1)
		{
			CString strName = strUserName.Left(nPos);
			int nBufLen = strName.GetLength();
			pUserList->pszName = new TCHAR[nBufLen + 1];
			if (NULL == pwszUserName)
			{
				SAFE_DELETE_ARR(pUserList->pszID);
				delete pUserList;
				goto DeleteMemory;
			}

			_tcscpy(pUserList->pszName, strName.GetBuffer());
			pUserList->pszName[nBufLen] = _T('\0');
			CString strNickName = strUserName.Mid(nPos + 1);
			pUserList->pszNickName = new TCHAR[strNickName.GetLength() + 1];
			if (NULL == pwszUserName)
			{
				SAFE_DELETE_ARR(pUserList->pszID);
				SAFE_DELETE_ARR(pUserList->pszName);
				delete pUserList;
				goto DeleteMemory;
			}
			wcscpy_s(pUserList->pszNickName, strNickName.GetLength() + 1, strNickName.GetBuffer());
			pUserList->pszNickName[strNickName.GetLength()] = _T('\0');
		}
		else
		{
			pUserList->pszName = NULL;
			pUserList->pszNickName = NULL;
		}		

		vecUserList.push_back(pUserList);		

		SAFE_DELETE_ARR(pszUserID);	
		SAFE_DELETE_ARR(pszUserName);
		SAFE_DELETE_ARR(pwszUserName);
	}	

	if (Read2Byte(nBodyLength, &stResJoin.nRoomType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &nJoinNum, &iterator) == FALSE)
		goto DeleteMemory;	

	if (Read4Byte(nBodyLength, &nLastMsgNum, &iterator) == FALSE)
		goto DeleteMemory;	

	if(stResJoin.wResCode == 0)
		strResult = L"SUCC";
	else
		strResult == L"FAIL";
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_JOIN:%s:PC:%s", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID(), strResult);

	if(m_pEvents)
		m_pEvents->NtalkClient_OnChatJoin(this, (NTalkChatJoinErrorCode)stResJoin.wResCode, stResJoin.nRoomID, vecUserList, stResJoin.nRoomType, nJoinNum, nLastMsgNum);

DeleteMemory:
	SAFE_DELETE_ARR(pszUserID);	
	SAFE_DELETE_ARR(pszUserName);
	SAFE_DELETE_ARR(pwszUserName);

	if(nUserCount > 0)
	{
		for(std::vector<PINVITE_USER>::iterator iter = vecUserList.begin(); iter != vecUserList.end(); iter++)
		{
			PINVITE_USER pInviteUser = (*iter);
			SAFE_DELETE_ARR(pInviteUser->pszID);
			SAFE_DELETE_ARR(pInviteUser->pszName);
			SAFE_DELETE_ARR(pInviteUser->pszNickName);

			delete (*iter);
		}
		vecUserList.clear();
	}	
}

void CNtalkClientSession::RecvResForceReJoin( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strResult;
	CString strStamp;
	CString strDbg;
	RES_JOIN stResJoin;
	USHORT nUserIdLen, nUserNameLen;
	char *pszUserID = NULL;
	char* pszUserName = NULL; //username+nickname
	std::vector<PINVITE_USER> vecUserList;
	TCHAR* pwszUserName = NULL;
	USHORT nUserCount = 0;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stResJoin, 0x00, sizeof(RES_JOIN));	

	const char* iterator = buffer;

	if (Read4Byte(nBodyLength, &stResJoin.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;	

	if (Read2Byte(nBodyLength, &stResJoin.wResCode, &iterator) == FALSE)
		goto DeleteMemory;	

	//	usercount | idlen | id | usernamelen | username+nickname| idlen | id|...
	//	1. ü ο ù° ̵
	//	usercount | idlen
	//	2. ù° ̵ ι° ̵ 
	//	id | idlen
	//	3. ...
	//	id | idlen
	//	4.  ̵
	//	id	
	if (Read2Byte(nBodyLength, &nUserCount, &iterator) == FALSE)
		goto DeleteMemory;
	for(int i = 0; i < nUserCount; i++)
	{
		PINVITE_USER pUserList = new INVITE_USER;
		if (NULL == pUserList)
			goto DeleteMemory;

		if (Read2Byte(nBodyLength, &nUserIdLen, &iterator) == FALSE)
			goto DeleteMemory;		

		// id
		if (ReadString(nBodyLength, &pszUserID, &iterator, nUserIdLen) == FALSE)
		{
			delete pUserList;
			goto DeleteMemory;
		}

		int nBufLen = MultiByteToWideChar(CP_UTF8, 0, pszUserID, -1, NULL, 0);		
		pUserList->pszID = new TCHAR[nBufLen];
		if (NULL == pUserList->pszID)
		{
			delete pUserList;
			goto DeleteMemory;
		}

		MultiByteToWideChar(CP_UTF8, 0, pszUserID, -1, pUserList->pszID, nBufLen);
		pUserList->pszID[nBufLen - 1] = _T('\0');

		// username + nickname
		if (Read2Byte(nBodyLength, &nUserNameLen, &iterator) == FALSE)
		{
			SAFE_DELETE_ARR(pUserList->pszID);		
			delete pUserList;
			goto DeleteMemory;
		}
		//  username+nickname Ŭ̾Ʈ  ãƼ ϱ⶧..׳ skip....		
		if (ReadString(nBodyLength, &pszUserName, &iterator, nUserNameLen) == FALSE)
		{
			SAFE_DELETE_ARR(pUserList->pszID);
			delete pUserList;
			goto DeleteMemory;
		}
		nBufLen = MultiByteToWideChar(CP_UTF8, 0, pszUserName, -1, NULL, 0);
		pwszUserName = new TCHAR[nBufLen];
		if (NULL == pwszUserName)
		{
			SAFE_DELETE_ARR(pUserList->pszID);
			delete pUserList;
			goto DeleteMemory;
		}
		MultiByteToWideChar(CP_UTF8, 0, pszUserName, -1, pwszUserName, nBufLen);		
		pwszUserName[nBufLen - 1] = _T('\0');

		CString strUserName(pwszUserName);
		int nPos = strUserName.Find(_T('|'));
		if(nPos != -1)
		{
			CString strName = strUserName.Left(nPos);
			pUserList->pszName = new TCHAR[strName.GetLength() + 1];
			if (NULL == pUserList->pszName)
			{
				SAFE_DELETE_ARR(pUserList->pszID);
				delete pUserList;
				goto DeleteMemory;
			}
			wcscpy_s(pUserList->pszName, strName.GetLength() + 1, strName.GetBuffer());
			pUserList->pszName[strName.GetLength()] = _T('\0');
			CString strNickName = strUserName.Mid(nPos + 1);
			pUserList->pszNickName = new TCHAR[strNickName.GetLength() + 1];
			if (NULL == pUserList->pszName)
			{
				SAFE_DELETE_ARR(pUserList->pszID);
				SAFE_DELETE_ARR(pUserList->pszName);
				delete pUserList;
				goto DeleteMemory;
			}
			wcscpy_s(pUserList->pszNickName, strNickName.GetLength() + 1, strNickName.GetBuffer());
			pUserList->pszNickName[strNickName.GetLength()] = _T('\0');
		}
		else
		{
			pUserList->pszName = NULL;
			pUserList->pszNickName = NULL;
		}

		vecUserList.push_back(pUserList);

		SAFE_DELETE_ARR(pszUserID);
		SAFE_DELETE_ARR(pszUserName);		
		SAFE_DELETE_ARR(pwszUserName);
	}
	
	if(stResJoin.wResCode == 0)
		strResult = L"SUCC";
	else
		strResult == L"FAIL";

	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_REJOIN:%s:PC:%s", strStamp,stResJoin.nRoomID, m_pNtalkModel->GetUserID(), strResult);	
	if(m_pEvents)
		m_pEvents->NtalkClient_OnForceChatReJoin(this, (NTalkChatJoinErrorCode)stResJoin.wResCode, stResJoin.nRoomID, vecUserList);

DeleteMemory:
	SAFE_DELETE_ARR(pszUserID);	
	SAFE_DELETE_ARR(pszUserName);
	SAFE_DELETE_ARR(pwszUserName);

	if(nUserCount > 0)
	{
		for(std::vector<PINVITE_USER>::iterator iter = vecUserList.begin(); iter != vecUserList.end(); iter++)
		{
			SAFE_DELETE_ARR((*iter)->pszID);
			SAFE_DELETE_ARR((*iter)->pszName);
			SAFE_DELETE_ARR((*iter)->pszNickName);

			delete (*iter);
		}		
		vecUserList.clear();
	}	
}

void CNtalkClientSession::RecvResForceLeave( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;
	CString strDbg;	
	RES_FORCELEAVE stResForceLeave;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();	
	int nBodyLength = pPacket->BodyLength();
	memset(&stResForceLeave, 0x00, sizeof(RES_FORCELEAVE));	

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stResForceLeave.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResForceLeave.nMyIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszMyID, &iterator, stResForceLeave.nMyIDLen) == FALSE)
		goto DeleteMemory;
	if (Read2Byte(nBodyLength, &stResForceLeave.nMyNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszMyNickName, &iterator, stResForceLeave.nMyNicknameLen) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResForceLeave.nMyOsType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResForceLeave.nUserIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszUserID, &iterator, stResForceLeave.nUserIDLen) == FALSE)
		goto DeleteMemory;
	if (Read2Byte(nBodyLength, &stResForceLeave.nNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszNickName, &iterator, stResForceLeave.nNicknameLen) == FALSE)
		goto DeleteMemory;



	TCHAR* pszUserID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResForceLeave.pszUserID, -1, NULL, 0);
	pszUserID = new TCHAR[nBufLen];
	if (NULL == pszUserID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResForceLeave.pszUserID, -1, pszUserID, nBufLen);
	pszUserID[nBufLen - 1] = _T('\0');


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_FORCE_LEAVE:%s:PC:%s", strStamp, stResForceLeave.nRoomID, m_pNtalkModel->GetUserID(), _T("SUCC"));	

	// Ŭ̾Ʈ  ൿ .
	//if(m_pEvents)
	//	m_pEvents->NtalkClient_OnForceChatLeave(this, forceLeaveError_None, stResForceLeave.nRoomID, pszUserID);

DeleteMemory:
	SAFE_DELETE_ARR(stResForceLeave.pszMyID);
	SAFE_DELETE_ARR(stResForceLeave.pszMyNickName);
	SAFE_DELETE_ARR(stResForceLeave.pszUserID);
	SAFE_DELETE_ARR(stResForceLeave.pszNickName);
	SAFE_DELETE_ARR(pszUserID);
}

void CNtalkClientSession::RecvPushForceLeave( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;
	CString strDbg;	
	RES_FORCELEAVE stResForceLeave;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();	
	int nBodyLength = pPacket->BodyLength();
	memset(&stResForceLeave, 0x00, sizeof(RES_FORCELEAVE));	

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stResForceLeave.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResForceLeave.nMyIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszMyID, &iterator, stResForceLeave.nMyIDLen) == FALSE)
		goto DeleteMemory;
	if (Read2Byte(nBodyLength, &stResForceLeave.nMyNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszMyNickName, &iterator, stResForceLeave.nMyNicknameLen) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResForceLeave.nMyOsType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResForceLeave.nUserIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszUserID, &iterator, stResForceLeave.nUserIDLen) == FALSE)
		goto DeleteMemory;
	if (Read2Byte(nBodyLength, &stResForceLeave.nNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResForceLeave.pszNickName, &iterator, stResForceLeave.nNicknameLen) == FALSE)
		goto DeleteMemory;



	TCHAR* pszUserID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResForceLeave.pszUserID, -1, NULL, 0);
	pszUserID = new TCHAR[nBufLen];
	if (NULL == pszUserID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResForceLeave.pszUserID, -1, pszUserID, nBufLen);
	pszUserID[nBufLen - 1] = _T('\0');


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_FORCE_LEAVE:%s:PC:%s", strStamp, stResForceLeave.nRoomID, m_pNtalkModel->GetUserID(), _T("SUCC"));	

	if(m_pEvents)
		m_pEvents->NtalkClient_OnPushForceLeave(this, stResForceLeave.nRoomID, pszUserID);

DeleteMemory:
	SAFE_DELETE_ARR(pszUserID);	
}

void CNtalkClientSession::RecvPushSessionClose( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;
	CString strDbg;	
	RES_FORCELEAVE stResForceLeave;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();	
	int nBodyLength = pPacket->BodyLength();
	memset(&stResForceLeave, 0x00, sizeof(RES_FORCELEAVE));	

	const char* iterator = buffer;		

	uint32_t nRoomID = 0;
	uint16_t nReasonCode = 0;

	if (Read4Byte(nBodyLength, &nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &nReasonCode, &iterator) == FALSE)
		goto DeleteMemory;

	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:SESSION_CLOSE:%s:PC:%d", strStamp, nRoomID, m_pNtalkModel->GetUserID(), nReasonCode);	

	if(m_pEvents)
		m_pEvents->NtalkClient_OnPushSessionClose(this, nRoomID);

DeleteMemory:
	;
}

void CNtalkClientSession::RecvPushNewJoiner( INpPacket * pPacket )
{
	CString strStamp;
	CString strDbg;	
	RES_NEWJOINER stResNewJoiner;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stResNewJoiner, 0x00, sizeof(RES_NEWJOINER));

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stResNewJoiner.nRoomID, &iterator) == FALSE)
		return;

	if (Read2Byte(nBodyLength, &stResNewJoiner.nUserIDLen, &iterator) == FALSE)
		return;

	if (ReadString(nBodyLength, &stResNewJoiner.pszUserID, &iterator, stResNewJoiner.nUserIDLen) == FALSE)
		return;

	TCHAR* pszRecvID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszUserID, -1, NULL, 0);
	pszRecvID = new TCHAR[nBufLen];
	if (NULL == pszRecvID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszUserID, -1, pszRecvID, nBufLen);
	pszRecvID[nBufLen - 1] = _T('\0');

	if (Read2Byte(nBodyLength, &stResNewJoiner.nNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;

	if (ReadString(nBodyLength, &stResNewJoiner.pszNickName, &iterator, stResNewJoiner.nNicknameLen) == FALSE)
		goto DeleteMemory;

	TCHAR* pszRecvNick = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszNickName, -1, NULL, 0);
	pszRecvNick = new TCHAR[nBufLen];
	if (NULL == pszRecvNick)
		goto DeleteMemory;
	MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszNickName, -1, pszRecvNick, nBufLen);
	pszRecvNick[nBufLen - 1] = _T('\0');

	if (Read2Byte(nBodyLength, &stResNewJoiner.nOsType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResNewJoiner.nMsgSeq, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResNewJoiner.nTime, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResNewJoiner.nMemberCount, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResNewJoiner.nJoinType, &iterator) == FALSE)
		stResNewJoiner.nJoinType = 1;


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_JOIN_ALARM:%s:PC:SUCC:%d", strStamp, stResNewJoiner.nRoomID, pszRecvID, stResNewJoiner.nJoinType);	
	if(m_pEvents)
		m_pEvents->NtalkClient_OnNewJoiner(this, stResNewJoiner.nRoomID, pszRecvID, pszRecvNick, stResNewJoiner.nTime, stResNewJoiner.nMemberCount, stResNewJoiner.nJoinType, stResNewJoiner.nOsType);

DeleteMemory:
	SAFE_DELETE_ARR(stResNewJoiner.pszUserID);
	SAFE_DELETE_ARR(stResNewJoiner.pszNickName);
	SAFE_DELETE_ARR(pszRecvID);
	SAFE_DELETE_ARR(pszRecvNick);	
}

void CNtalkClientSession::RecvPushForceReJoiner( INpPacket * pPacket )
{
	CString strStamp;
	CString strDbg;
	RES_NEWJOINER stResNewJoiner;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();	
	int nBodyLength = pPacket->BodyLength();
	memset(&stResNewJoiner, 0x00, sizeof(RES_NEWJOINER));

	const char* iterator = buffer;

	if (Read4Byte(nBodyLength, &stResNewJoiner.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResNewJoiner.nUserIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResNewJoiner.pszUserID, &iterator, stResNewJoiner.nUserIDLen) == FALSE)
		goto DeleteMemory;
	TCHAR* pszRecvID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszUserID, -1, NULL, 0);
	pszRecvID = new TCHAR[nBufLen];
	if (NULL == pszRecvID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszUserID, -1, pszRecvID, nBufLen);
	pszRecvID[nBufLen - 1] = _T('0');

	if (Read2Byte(nBodyLength, &stResNewJoiner.nNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResNewJoiner.pszNickName, &iterator, stResNewJoiner.nNicknameLen) == FALSE)
		goto DeleteMemory;
	TCHAR* pszRecvNick = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszNickName, -1, NULL, 0);
	pszRecvNick = new TCHAR[nBufLen];
	if (NULL == pszRecvNick)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResNewJoiner.pszNickName, -1, pszRecvNick, nBufLen);
	pszRecvNick[nBufLen - 1] = _T('\0');

	if (Read2Byte(nBodyLength, &stResNewJoiner.nOsType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResNewJoiner.nMsgSeq, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResNewJoiner.nTime, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResNewJoiner.nMemberCount, &iterator) == FALSE)
		goto DeleteMemory;


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_REJOIN_ALARM:%s:PC:SUCC", strStamp,stResNewJoiner.nRoomID, pszRecvID);	
	if(m_pEvents)
		m_pEvents->NtalkClient_OnForceReJoiner(this, stResNewJoiner.nRoomID, pszRecvID, pszRecvNick, stResNewJoiner.nTime, stResNewJoiner.nMemberCount);

DeleteMemory:
	SAFE_DELETE_ARR(stResNewJoiner.pszUserID);
	SAFE_DELETE_ARR(stResNewJoiner.pszNickName);
	SAFE_DELETE_ARR(pszRecvID);
	SAFE_DELETE_ARR(pszRecvNick);	
}

void CNtalkClientSession::RecvPushLeaveJoiner( INpPacket * pPacket )
{
	CString strStamp;
	CString strDbg;	
	RES_LEAVEJOINER stResLeaveJoiner;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();	
	int nBodyLength = pPacket->BodyLength();
	memset(&stResLeaveJoiner, 0x00, sizeof(RES_LEAVEJOINER));	

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stResLeaveJoiner.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;
	if (Read2Byte(nBodyLength, &stResLeaveJoiner.nUserIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResLeaveJoiner.pszUserID, &iterator, stResLeaveJoiner.nUserIDLen) == FALSE)
		goto DeleteMemory;

	TCHAR* pszRecvID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResLeaveJoiner.pszUserID, -1, NULL, 0);
	pszRecvID = new TCHAR[nBufLen];
	if (NULL == pszRecvID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResLeaveJoiner.pszUserID, -1, pszRecvID, nBufLen);
	pszRecvID[nBufLen - 1] = _T('\0');

	if (Read2Byte(nBodyLength, &stResLeaveJoiner.nNicknameLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stResLeaveJoiner.pszNickName, &iterator, stResLeaveJoiner.nNicknameLen) == FALSE)
		goto DeleteMemory;
	TCHAR* pszRecvNick = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stResLeaveJoiner.pszNickName, -1, NULL, 0);
	pszRecvNick = new TCHAR[nBufLen];
	if (NULL == pszRecvNick)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stResLeaveJoiner.pszNickName, -1, pszRecvNick, nBufLen);
	pszRecvNick[nBufLen - 1] = _T('\0');

	if (Read2Byte(nBodyLength, &stResLeaveJoiner.nOsType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResLeaveJoiner.nMsgSeq, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResLeaveJoiner.nTime, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResLeaveJoiner.nMemberCount, &iterator) == FALSE)
		goto DeleteMemory;


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_LEAVE_ALARM:%s:%s:SUCC", strStamp, stResLeaveJoiner.nRoomID, pszRecvID, stResLeaveJoiner.nOsType == 1 ? L"PC" : L"MOBILE");
	if(m_pEvents)
	{		
		m_pEvents->NtalkClient_OnChatLeaveJoiner(this, 
			stResLeaveJoiner.nRoomID,
			pszRecvID,
			pszRecvNick, 
			stResLeaveJoiner.nOsType, 
			stResLeaveJoiner.nTime,
			stResLeaveJoiner.nMemberCount);
	}

DeleteMemory:
	SAFE_DELETE_ARR(stResLeaveJoiner.pszUserID);
	SAFE_DELETE_ARR(stResLeaveJoiner.pszNickName);
	SAFE_DELETE_ARR(pszRecvID);
	SAFE_DELETE_ARR(pszRecvNick);	
}

void CNtalkClientSession::RecvAckSendMsg( INpPacket * pPacket )
{
	CString strStamp;
	CString strDbg;
	ACK_SENDMSG stAckSendMsg;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stAckSendMsg, 0x00, sizeof(ACK_SENDMSG));	

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stAckSendMsg.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stAckSendMsg.nMsgSeq, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stAckSendMsg.nUserIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stAckSendMsg.pszUserID, &iterator, stAckSendMsg.nUserIDLen) == FALSE)
		goto DeleteMemory;
	TCHAR* pszRecvID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stAckSendMsg.pszUserID, -1, NULL, 0);
	pszRecvID = new TCHAR[nBufLen];
	if (NULL == pszRecvID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stAckSendMsg.pszUserID, -1, pszRecvID, nBufLen);
	pszRecvID[nBufLen - 1] = _T('\0');


	if (Read4Byte(nBodyLength, &stAckSendMsg.nTid, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stAckSendMsg.nTime, &iterator) == FALSE)
		goto DeleteMemory;


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:%d:MSG_ACK:%s:PC:SUCC", strStamp, stAckSendMsg.nRoomID, stAckSendMsg.nMsgSeq, pszRecvID);

	if(m_pEvents)
	{
		m_pEvents->NtalkClient_OnAckSendMsg(this, 
			stAckSendMsg.nRoomID,
			stAckSendMsg.nMsgSeq,
			pszRecvID,
			stAckSendMsg.nTid,
			stAckSendMsg.nTime);
	}

DeleteMemory:
	SAFE_DELETE_ARR(stAckSendMsg.pszUserID);
	SAFE_DELETE_ARR(pszRecvID);
}

void CNtalkClientSession::RecvAckSendFile( INpPacket * pPacket )
{
	CString strStamp;
	CString strDbg;
	ACK_SENDMSG stAckSendMsg;
	int nBufLen = 0;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stAckSendMsg, 0x00, sizeof(ACK_SENDMSG));	

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stAckSendMsg.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stAckSendMsg.nMsgSeq, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stAckSendMsg.nUserIDLen, &iterator) == FALSE)
		goto DeleteMemory;
	if (ReadString(nBodyLength, &stAckSendMsg.pszUserID, &iterator, stAckSendMsg.nUserIDLen) == FALSE)
		goto DeleteMemory;
	TCHAR* pszRecvID = NULL;
	nBufLen = MultiByteToWideChar(CP_UTF8, 0, stAckSendMsg.pszUserID, -1, NULL, 0);
	pszRecvID = new TCHAR[nBufLen];
	if (NULL == pszRecvID)
		goto DeleteMemory;

	MultiByteToWideChar(CP_UTF8, 0, stAckSendMsg.pszUserID, -1, pszRecvID, nBufLen);
	pszRecvID[nBufLen - 1] = _T('\0');


	if (Read4Byte(nBodyLength, &stAckSendMsg.nTid, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stAckSendMsg.nTime, &iterator) == FALSE)
		goto DeleteMemory;


	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:%d:MSG_ACK:%s:PC:SUCC", strStamp, stAckSendMsg.nRoomID, stAckSendMsg.nMsgSeq, pszRecvID);

	if(m_pEvents)
	{
		m_pEvents->NtalkClient_OnAckSendFile(this, 
			stAckSendMsg.nRoomID,
			stAckSendMsg.nMsgSeq,
			pszRecvID,
			stAckSendMsg.nTid,
			stAckSendMsg.nTime);
	}

DeleteMemory:
	SAFE_DELETE_ARR(stAckSendMsg.pszUserID);
	SAFE_DELETE_ARR(pszRecvID);
}

void CNtalkClientSession::RecvResInitRoom( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;	
	RES_INITROOM stResInitRoom;

	const char* buffer;
	memset(&stResInitRoom, 0x00, sizeof(RES_INITROOM));	

	buffer = pPacket->BodyData();
	int nBodyLength  = pPacket->BodyLength();

	memcpy(&stResInitRoom, buffer, nBodyLength);

	CString strResult;
	if(stResInitRoom.wResCode == 0)
		strResult = L"SUCC";
	else
		strResult == L"FAIL";

	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:ROOM_CREATE:%s:PC:%s", strStamp, stResInitRoom.nRoomID, m_pNtalkModel->GetUserID(), strResult);
	if(m_pEvents)
		m_pEvents->NtalkClient_OnInitRoom(this, stResInitRoom.nRoomID, stResInitRoom.wResCode == 0 ? createSessionError_None : createSessionError_InitRoom);
}

void CNtalkClientSession::RecvResInviteList( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strResult;
	CString strStamp;
	RES_INVITELIST stResInviteList;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stResInviteList, 0x00, sizeof(RES_INVITELIST));	

	const char* iterator = buffer;

	if (Read4Byte(nBodyLength, &stResInviteList.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResInviteList.wResCode, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResInviteList.nTotalMemberCount, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResInviteList.nTryCount, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResInviteList.nFailCount, &iterator) == FALSE)
		goto DeleteMemory;

	int nUserIDLen = 0;
	char* szUserID = NULL;

	for(UINT i = 0; i < stResInviteList.nFailCount; i++)
	{
		PINVITE_USER pInviteUser = new INVITE_USER;
		if (NULL == pInviteUser)
			goto DeleteMemory;

		if (Read2Byte(nBodyLength, &nUserIDLen, &iterator) == FALSE)
		{
			delete pInviteUser;
			goto DeleteMemory;
		}

		if (ReadString(nBodyLength, &szUserID, &iterator, nUserIDLen) == FALSE)
		{
			delete pInviteUser;
			goto DeleteMemory;
		}

		int nBufLen = MultiByteToWideChar(CP_UTF8, 0, szUserID, -1, NULL, 0);
		TCHAR *pszID = new TCHAR[nBufLen];
		if (NULL == pszID)
		{
			delete pInviteUser;
			goto DeleteMemory;
		}

		MultiByteToWideChar(CP_UTF8, 0, szUserID, -1, pszID, nBufLen);
		pszID[nBufLen - 1] = _T('\0');

		SAFE_DELETE_ARR(szUserID);

		stResInviteList.vecUserIDList.push_back(pszID);		
	}

	
	if(stResInviteList.wResCode == 0)
		strResult = L"SUCC";
	else
		strResult == L"FAIL";
	
	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_ADD:%s:PC:%s", strStamp, stResInviteList.nRoomID, m_pNtalkModel->GetUserID(), strResult);
	if(m_pEvents)
	{		
		m_pEvents->NtalkClient_OnInviteList(this, 
											stResInviteList.nRoomID,
											(NTalkInviteListErrorCode)stResInviteList.wResCode,
											stResInviteList.nTotalMemberCount,
											stResInviteList.nTryCount,
											stResInviteList.nFailCount, 
											stResInviteList.vecUserIDList);
	}

DeleteMemory:

	if(stResInviteList.nFailCount > 0)
	{
		std::vector<TCHAR*>::iterator iter = stResInviteList.vecUserIDList.begin();
		for(; iter != stResInviteList.vecUserIDList.end(); iter++)
		{
			SAFE_DELETE_ARR((*iter));
		}

		stResInviteList.vecUserIDList.clear();	
	}
	SAFE_DELETE_ARR(szUserID);	
}

void CNtalkClientSession::RecvPushRoomEnd( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	CString strStamp;
	RES_ROOMEND stResRoomEnd;

	const char* buffer = pPacket->BodyData();	
	int nBodyLength = pPacket->BodyLength();
	memset(&stResRoomEnd, 0x00, sizeof(RES_ROOMEND));

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stResRoomEnd.nRoomID, &iterator) == FALSE)
		return;

	if (Read2Byte(nBodyLength, &stResRoomEnd.nType, &iterator) == FALSE)
		return;

	LogUtil::GetCurrentTimeStamp(strStamp);
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:ROOM_END:%s:PC::%s", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID(), stResRoomEnd.nType == 1 ? L"TIMEOUT" : L"EMPTY");	
	if(m_pEvents)
	{
		m_pEvents->NtalkClient_OnRoomEnd(this, stResRoomEnd.nRoomID, stResRoomEnd.nType);
	}
}

void CNtalkClientSession::RecvPushSessionEnd( INpPacket * pPacket )
{
    if( m_pNtalkModel == NULL )
        return;

    CString strDbg;	
    RES_SESSIONEND stSessionEnd;
    int nBufLen = 0;
    CString strStamp;

    const char* buffer;	
    memset(&stSessionEnd, 0x00, sizeof(RES_SESSIONEND));

    buffer = pPacket->BodyData();
    int nBodyLength = pPacket->BodyLength();

    const char* iterator = buffer;
    if (Read4Byte(nBodyLength, &stSessionEnd.nRoomID, &iterator) == FALSE)
        goto DeleteMemory;

    if (Read2Byte(nBodyLength, &stSessionEnd.nUserLen, &iterator) == FALSE)
        goto DeleteMemory;

    if (ReadString(nBodyLength, &stSessionEnd.pszUserID, &iterator, stSessionEnd.nUserLen) == FALSE)
        goto DeleteMemory;
    if (Read2Byte(nBodyLength, &stSessionEnd.nNicknameLen, &iterator) == FALSE)
        goto DeleteMemory;

    TCHAR* pszRecvID = NULL;
    nBufLen = MultiByteToWideChar(CP_UTF8, 0, stSessionEnd.pszUserID, -1, NULL, 0);
    pszRecvID = new TCHAR[nBufLen];
    if (NULL == pszRecvID)
        goto DeleteMemory;

    MultiByteToWideChar(CP_UTF8, 0, stSessionEnd.pszUserID, -1, pszRecvID, nBufLen);
    pszRecvID[nBufLen - 1] = _T('\0');

    if (ReadString(nBodyLength, &stSessionEnd.pszNickName, &iterator, stSessionEnd.nNicknameLen) == FALSE)
        goto DeleteMemory;

    if (Read2Byte(nBodyLength, &stSessionEnd.nOsType, &iterator) == FALSE)
        goto DeleteMemory;

    TCHAR* pszRecvNick = NULL;
    nBufLen = MultiByteToWideChar(CP_UTF8, 0, stSessionEnd.pszNickName, -1, NULL, 0);
    pszRecvNick = new TCHAR[nBufLen];
    if (NULL == pszRecvNick)
        goto DeleteMemory;

    MultiByteToWideChar(CP_UTF8, 0, stSessionEnd.pszNickName, -1, pszRecvNick, nBufLen);
    pszRecvNick[nBufLen - 1] = _T('\0');


    LogUtil::GetCurrentTimeStamp(strStamp);
    //GT_FLOG(L"[%s]:P:RCV:%d:-1:USER_SESSION_END_ALARM:%s:PC:SUCC:%d", strStamp, m_pNtalkModel->GetRoomID(), pszRecvID);
    if(m_pEvents)
    {		
        m_pEvents->NtalkClient_OnSessionEnd(this, stSessionEnd.nRoomID, pszRecvID, pszRecvNick, (int)stSessionEnd.nOsType);
    }

DeleteMemory:
    SAFE_DELETE_ARR(stSessionEnd.pszUserID);
    SAFE_DELETE_ARR(stSessionEnd.pszNickName);
    SAFE_DELETE_ARR(pszRecvID);
    SAFE_DELETE_ARR(pszRecvNick);
}

void CNtalkClientSession::RecvServerNotReady()
{
	if( m_pNtalkModel == NULL )
		return;

	//CString strStamp;
	//GT_FLOG(L"[%s]:P:RCV:%d:-1:SERVER_NOT_READY:%s:PC:", strStamp, m_pNtalkModel->GetRoomID(), m_pNtalkModel->GetUserID());	
	//GT_FLOG_NOARG(L"SERVER_NOT_READY");

	if(m_pEvents)
	{
		m_pEvents->NtalkClient_OnServerNotReady(this);
	}
}

/************************************************************************/
/*  ŵ Ŷ                                                 */
/************************************************************************/
void CNtalkClientSession::RecvPushMsgFail( INpPacket * pPacket )
{
	RES_MSGFAIL stResMsgFail;
	char* szUserID = NULL;
	char* szUserName = NULL;
	int nBufLen = 0;
	std::vector<COMM_USERINFO *> vecFailUser;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stResMsgFail, 0x00, sizeof(RES_MSGFAIL));

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stResMsgFail.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgFail.nMsgSeq, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgFail.nTid, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgFail.nTime, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResMsgFail.nOsType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgFail.nSendFailUsersCount, &iterator) == FALSE)
		goto DeleteMemory;	


	for(UINT i = 0; i < stResMsgFail.nSendFailUsersCount; i++)
	{
		COMM_USERINFO *pstCommUserInfo = new COMM_USERINFO;
		if (NULL == pstCommUserInfo)
			goto DeleteMemory;

		if (Read2Byte(nBodyLength, &(pstCommUserInfo->nUserIdLen), &iterator) == FALSE)
		{
			delete pstCommUserInfo;
			goto DeleteMemory;
		}
		if (ReadString(nBodyLength, &szUserID, &iterator, pstCommUserInfo->nUserIdLen) == FALSE)
		{
			delete pstCommUserInfo;
			goto DeleteMemory;
		}
		nBufLen = MultiByteToWideChar(CP_UTF8, 0, szUserID, -1, NULL, 0);
		pstCommUserInfo->pszUserID = new TCHAR[nBufLen];
		if (NULL == pstCommUserInfo->pszUserID)
		{
			delete pstCommUserInfo;
			goto DeleteMemory;
		}

		MultiByteToWideChar(CP_UTF8, 0, szUserID, -1, pstCommUserInfo->pszUserID, nBufLen);
		pstCommUserInfo->pszUserID[nBufLen - 1] = _T('\0');		

		if (Read2Byte(nBodyLength, &(pstCommUserInfo->nUserNameLength), &iterator) == FALSE)
			goto DeleteMemory;		
		if (ReadString(nBodyLength, &szUserName, &iterator, pstCommUserInfo->nUserNameLength) == FALSE)
			goto DeleteMemory;		
		nBufLen = MultiByteToWideChar(CP_UTF8, 0, szUserName, -1, NULL, 0);
		pstCommUserInfo->pszUserName = new TCHAR[nBufLen];
		if (NULL == pstCommUserInfo->pszUserName)
		{
			SAFE_DELETE_ARR(pstCommUserInfo->pszUserID);
			delete pstCommUserInfo;
			goto DeleteMemory;
		}
		MultiByteToWideChar(CP_UTF8, 0, szUserName, -1, pstCommUserInfo->pszUserName, nBufLen);
		pstCommUserInfo->pszUserName[nBufLen - 1] = _T('\0');

		vecFailUser.push_back(pstCommUserInfo);

		SAFE_DELETE_ARR(szUserID);
		SAFE_DELETE_ARR(szUserName);		
	}

	if(m_pEvents)
	{
		m_pEvents->NtalkClient_OnSendMsgFail(this, stResMsgFail.nRoomID, stResMsgFail.nMsgSeq, stResMsgFail.nTid, stResMsgFail.nTime, 
			stResMsgFail.nOsType, vecFailUser);
	}

DeleteMemory:	
	SAFE_DELETE_ARR(szUserID);
	SAFE_DELETE_ARR(szUserName);

	if(stResMsgFail.nSendFailUsersCount > 0)
	{
		for(std::vector<COMM_USERINFO *>::iterator iter = vecFailUser.begin(); iter != vecFailUser.end(); iter++)
		{
			SAFE_DELETE_ARR((*iter)->pszUserID);
			SAFE_DELETE_ARR((*iter)->pszUserName);			

			delete (*iter);
		}		
		vecFailUser.clear();
	}	
}

void CNtalkClientSession::RecvPushMsgResult( INpPacket * pPacket )
{
	if( m_pNtalkModel == NULL )
		return;

	RES_MSGRESULT stResMsgResult;
	char* szUserID = NULL;
	char* szUserName = NULL;
	int nBufLen = 0;
	std::vector<SEND_MSGRESULT *> vecUserMsgResult;
	TCHAR* pwszUserName = NULL;

	const char* buffer = pPacket->BodyData();
	int nBodyLength = pPacket->BodyLength();
	memset(&stResMsgResult, 0x00, sizeof(RES_MSGRESULT));

	const char* iterator = buffer;		

	if (Read4Byte(nBodyLength, &stResMsgResult.nRoomID, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgResult.nMsgSeq, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgResult.nTid, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgResult.nTime, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read2Byte(nBodyLength, &stResMsgResult.nOsType, &iterator) == FALSE)
		goto DeleteMemory;

	if (Read4Byte(nBodyLength, &stResMsgResult.nResultSeq, &iterator) == FALSE)
		goto DeleteMemory;
	if (Read4Byte(nBodyLength, &stResMsgResult.nResultMsgUserCount, &iterator) == FALSE)
		goto DeleteMemory;


	for(UINT i = 0; i < stResMsgResult.nResultMsgUserCount; i++)
	{
		SEND_MSGRESULT *pstSendMsgResult = new SEND_MSGRESULT;
		if (NULL == pstSendMsgResult)
			goto DeleteMemory;

		if (Read2Byte(nBodyLength, &(pstSendMsgResult->nUserIdLen), &iterator) == FALSE)
		{
			delete pstSendMsgResult;
			goto DeleteMemory;
		}
		if (ReadString(nBodyLength, &szUserID, &iterator, pstSendMsgResult->nUserIdLen) == FALSE)
		{
			delete pstSendMsgResult;
			goto DeleteMemory;
		}

		nBufLen = MultiByteToWideChar(CP_UTF8, 0, szUserID, -1, NULL, 0);
		pstSendMsgResult->pszUserID = new TCHAR[nBufLen];
		if (NULL == pstSendMsgResult->pszUserID)
		{
			delete pstSendMsgResult;
			goto DeleteMemory;
		}

		MultiByteToWideChar(CP_UTF8, 0, szUserID, -1, pstSendMsgResult->pszUserID, nBufLen);
		pstSendMsgResult->pszUserID[nBufLen - 1] = _T('\0');		


		if (Read2Byte(nBodyLength, &(pstSendMsgResult->nUserNameLength), &iterator) == FALSE)
		{
			delete pstSendMsgResult;
			goto DeleteMemory;
		}


		if (ReadString(nBodyLength, &szUserName, &iterator, pstSendMsgResult->nUserNameLength) == FALSE)
		{
			delete pstSendMsgResult;
			goto DeleteMemory;
		}

		nBufLen = MultiByteToWideChar(CP_UTF8, 0, szUserName, -1, NULL, 0);
		pwszUserName = new TCHAR[nBufLen];
		if (NULL == pwszUserName)
		{
			SAFE_DELETE_ARR(pstSendMsgResult->pszUserID);
			delete pstSendMsgResult;
			goto DeleteMemory;
		}
		MultiByteToWideChar(CP_UTF8, 0, szUserName, -1, pwszUserName, nBufLen);
		pwszUserName[nBufLen - 1] = _T('\0');
		CString strUserName(pwszUserName);
		pstSendMsgResult->pszUserName = new TCHAR[strUserName.GetLength() + 1];
		wcscpy_s(pstSendMsgResult->pszUserName, strUserName.GetLength() + 1, strUserName.GetBuffer());
		SAFE_DELETE_ARR(pwszUserName);


		if (Read2Byte(nBodyLength, &(pstSendMsgResult->nUserOsType), &iterator) == FALSE)
		{
			SAFE_DELETE_ARR(pstSendMsgResult->pszUserID);
			delete pstSendMsgResult;
			goto DeleteMemory;
		}

		if (Read4Byte(nBodyLength, &(pstSendMsgResult->nResult), &iterator) == FALSE)
		{
			SAFE_DELETE_ARR(pstSendMsgResult->pszUserID);
			delete pstSendMsgResult;
			goto DeleteMemory;
		}

		CString strStamp;
		LogUtil::GetCurrentTimeStamp(strStamp);
		CString strResult;
		if(pstSendMsgResult->nResult == 1)
			strResult = L"SUCC";
		else
			strResult == L"FAIL";
		//Ÿ  ϰ ߱ ؼ NONE ߰
		CString strOSType;
		if(pstSendMsgResult->nUserOsType == 0)
			strOSType = L"NONE";
		else if(pstSendMsgResult->nUserOsType == 1)
			strOSType = L"PC";
		else if(pstSendMsgResult->nUserOsType == 2)
			strOSType = L"Mobile";
		//GT_FLOG(L"[%s]:P:RCV:%d:%d:MSG_RESULT:%s:%s:%s:%s:%d", strStamp, m_pNtalkModel->GetRoomID(), stResMsgResult.nMsgSeq, pstSendMsgResult->pszUserID, strOSType, L"SUCC", strResult, stResMsgResult.nResultSeq);

		vecUserMsgResult.push_back(pstSendMsgResult);

		SAFE_DELETE_ARR(szUserID);
		SAFE_DELETE_ARR(szUserName);
	}	

	if(m_pEvents)
	{
		RequestAckMsgResult(stResMsgResult.nMsgSeq, stResMsgResult.nTid, stResMsgResult.nResultSeq);
		m_pEvents->NtalkClient_OnMsgResult(this, stResMsgResult.nRoomID, stResMsgResult.nMsgSeq, 
			stResMsgResult.nTid, stResMsgResult.nTime, stResMsgResult.nOsType, 
			stResMsgResult.nResultSeq, vecUserMsgResult);

	}

DeleteMemory:	
	SAFE_DELETE_ARR(szUserID);
	SAFE_DELETE_ARR(szUserName);

	if(stResMsgResult.nResultMsgUserCount > 0)
	{
		for(std::vector<SEND_MSGRESULT *>::iterator iter = vecUserMsgResult.begin(); iter != vecUserMsgResult.end(); iter++)
		{
			SEND_MSGRESULT* pMsgResult = *iter;
			SAFE_DELETE_ARR(pMsgResult->pszUserID);
			SAFE_DELETE_ARR(pMsgResult->pszUserName);			

			SAFE_DELETE(*iter);			
		}		
		vecUserMsgResult.clear();
	}
}

void CNtalkClientSession::RecvPong()
{
	m_pEvents->NtalkClient_OnRecvPong();
}

int CNtalkClientSession::EncryptMsg( IN LPCTSTR strID, IN char* szPlainMsg, OUT unsigned char* szEncryptedMsg )
{
	CAESCrypto crypto;
	char szKeyTable[] = 
		"0ZO67A8d60SM0uZvZlU347DGHmVt9505pxh447U2gx8BNO342r6gin4STd9t9x7A2qHI8ukxDyw9nG5o2491xg82w707MMrdS1Zrn8QAP3Za23F4E8HsxVWTyNuFPmFNPc2dlInGyBR6w2s80Nfx16";
	char *pszAnsiID = NULL;
	char szSeed[1024] = {0};
	//char szEncryptedSip[1024] = {0};

	int nPwLen = 0;
	int nB64Len = 0;

	USES_CONVERSION;

	pszAnsiID = W2A(strID);
	crypto.Base64Encode((const BYTE*)pszAnsiID, strlen(pszAnsiID), szSeed, &nB64Len);	

	bool result = crypto.pwdcrypt_encrypt(szPlainMsg, strlen(szPlainMsg), (unsigned char *)szEncryptedMsg, &nPwLen, szSeed, szKeyTable, strlen(szKeyTable));	
	if(result == false)
	{
		nPwLen = 0;
	}

	return nPwLen;
}

void CNtalkClientSession::DecryptMsg( IN LPCTSTR strID, IN unsigned char* szEncMsg, IN int nEncSize, OUT char* szPlainMsg )
{
	CAESCrypto crypto;
	char szKeyTable[] = 
		"0ZO67A8d60SM0uZvZlU347DGHmVt9505pxh447U2gx8BNO342r6gin4STd9t9x7A2qHI8ukxDyw9nG5o2491xg82w707MMrdS1Zrn8QAP3Za23F4E8HsxVWTyNuFPmFNPc2dlInGyBR6w2s80Nfx16";

	char *pszAnsiID = NULL;
	char szSeed[1024] = {0,};
	int nB64Len = 0;
	int nDecLen = 0;

	USES_CONVERSION;

	pszAnsiID = W2A(strID);
	crypto.Base64Encode((const BYTE*)pszAnsiID, strlen(pszAnsiID), szSeed, &nB64Len);	

	crypto.pwdcrypt_decrypt((unsigned char *)szEncMsg, nEncSize, szPlainMsg, &nDecLen, szSeed, szKeyTable, strlen(szKeyTable));
}

BOOL CNtalkClientSession::RequestReCoverSession()
{
	if( m_pNtalkModel == NULL )
		return FALSE;

	bool bSuccess = false;
	if(m_pNpSession)
	{
		bSuccess = m_pNpSession->ConnectSession(m_pNtalkModel->GetAddress(), m_pNtalkModel->GetPort());
	}

	return (BOOL)bSuccess;
}

void CNtalkClientSession::ReleaseSession()
{
	if(m_pNpSession)
	{
		m_pNpSession->CloseSession();
		m_pNpSession->Delete();
		m_pNpSession = NULL;
	}
}
